package com.ablecloud.demo;

import com.ablecloud.common.*;
import com.ablecloud.service.AC;
import com.ablecloud.service.ACDStore;
import com.ablecloud.service.ACStore;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.FixMethodOrder;
import org.junit.runners.MethodSorters;
import org.junit.Test;

import java.util.Iterator;
import java.util.List;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class DemoServiceTest {
    // 记录起始开始测试的时间
    private static long startTime;
    // 我们会在多个测试case中用到以下的成员，并且只需要初始化一次，
    // 因此我们定义为static的
    private static ACConfiguration config;  // 测试过程的配置信息，在cloudservice-conf.xml中配置
    private static AC ac;                   // 测试用的AC框架，正式环境由框架初始化，开发者不用关注
    private static DemoService demoService; // 我们开发的服务
    private static ACAccount account;       // 用于保存测试的帐号信息
    private static ACUserDevice light;      // 用于保存测试的设备信息
    private static String subDomainName;

    // 在所有的test case执行前初始化一次
    @BeforeClass
    public static void setUp() throws Exception {
        try {
            // 这里初始化config对象
            config = new ACConfiguration("./package/config/cloudservice-conf.xml");
            ac = AC.getTestAc(config);          // 通过AC的接口获取用于测试的ac框架

            demoService = new DemoService();
            demoService.setEnv(ac);     // 需要调用该接口，将框架ac赋予demoService
            demoService.init();                 // 初始化demoService
            
            subDomainName = config.getSubDomain();

            // 使用开发者权限创建一个测试账号
            try {
                account = ac.accountMgrForTest(ac.newContext("")).register("test@ablecloud.cn", "13100000000", "pswd");
            } catch (ACServiceException e) {
                if (e.getErrorCode() == 3502) {
                    // 帐号已注册
                    account = ac.accountMgrForTest(ac.newContext("")).login("test@ablecloud.cn", "pswd");
                } else {
                    throw e;
                }
            }

            // 使用注册的测试账号绑定一个虚拟的测试设备
            light = ac.bindMgrForTest(ac.newContext(subDomainName)).bindDevice(subDomainName, "12345678", "light1", account.getUid());
            // 记录起始开始测试的时间
            startTime = System.currentTimeMillis();
        } catch (Exception e) {
            e.printStackTrace();
            fail("set up fail");
        }
    }


    @AfterClass
    public static void tearDown() throws Exception {
        // 执行完test后，需要解绑setUp绑定的测试设备，确保下次单测能顺利通过
        if (light != null && account != null) {
            ac.bindMgrForTest(ac.newContext(subDomainName)).unbindDevice(subDomainName, light.getId(), account.getUid());
        }
    }

    @Test
    public void test1ControlLight() throws Exception {
        if (account == null || light == null)
            return;

        try {
            // 创建一个用户的context
            ACContext context = ac.newContext(subDomainName, account.getUid());
            // 添加一个灯的桩
            ac.addDeviceStub(subDomainName, new LightStub());

            // 下面构造client发送的请求参数
            ACMsg req = new ACMsg();
            req.setContext(context);            // 将上面用户的context传入
            req.setName("controlLight");        // 设置请求消息的名字
            req.put("deviceId", light.getId()); // 设置要控制灯的逻辑id
            req.put("action", "on");            // "on"表示开灯

            // 构造响应消息，用于接收服务端返回的消息
            ACMsg resp = new ACMsg();
            // 这里直接调用服务的处理handler，驱动测试
            demoService.handleMsg(req, resp);
            // 服务发送消息给设备后，设备会将处理结果代码返回
            // 在我们实现的LightStub中，比较结果是否正确
            assertEquals("success", resp.get("result"));
            // 成功打开灯之后，模拟设备进行数据上报
            try {
                ACDeviceMsg deviceMsg = new ACDeviceMsg(LightMsg.REPORT_CODE, new byte[]{LightMsg.ON, LightMsg.FROM_APP});
                // 这里直接调用服务的设备消息处理handler，驱动测试
                ACDeviceReportInfo reportInfo = new ACDeviceReportInfo();
                reportInfo.setContext(ac.newContext(subDomainName));
                reportInfo.setDeviceId(light.getId());
                reportInfo.setPhysicalDeviceId(light.getPhysicalId());
                demoService.handleDeviceMsg(reportInfo, deviceMsg);
            } catch (Exception e) {
                e.printStackTrace();
                fail("test light report fail");
            }
        } catch (ACServiceException e) {
            e.printStackTrace();
            fail("test control light fail");
        }
    }

    @Test
    public void test2LightReportAndQuery() throws Exception {
        if (account == null || light == null)
            return;

        // 先测试智能灯上报消息，将上报数据写入设备属性中
        try {
            ACDeviceMsg acDeviceMsg = new ACDeviceMsg(LightMsg.REPORT_CODE, new byte[]{LightMsg.ON, LightMsg.FROM_SWITCH});
            // 这里直接调用服务的设备消息处理handler，驱动测试
            ACDeviceReportInfo reportInfo = new ACDeviceReportInfo();
            reportInfo.setContext(ac.newContext(subDomainName));
            reportInfo.setDeviceId(light.getId());
            reportInfo.setPhysicalDeviceId(light.getPhysicalId());
            demoService.handleDeviceMsg(reportInfo, acDeviceMsg);
        } catch (ACServiceException e) {
            e.printStackTrace();
            fail("test light report fail");
        }

        // 这里用上面写入设备属性存储的数据来驱动测试app发来的数据查询请求处理handler
        try {
            List<ACObject> historyDatas = ac.dstore(subDomainName)
                    .scanHistory(light.getId())
                    .startTime(true, startTime) //true为代表闭区间包含的意思
                    .endTime(true, System.currentTimeMillis())
                    .execute();
            if (historyDatas != null) {
                assertEquals(2, historyDatas.size());
                for (ACObject zo : historyDatas) {
                    System.out.println(zo.toString());
                    //获取设备的逻辑ID
                    long deviceId = zo.getLong(ACDStore.DEVICE_ID);
                    //获取设备属性历史记录的时间戳
                    long timestamp = zo.getLong(ACDStore.TIMESTAMP);
                    assertEquals(deviceId, light.getId());
                    assertTrue(timestamp >= startTime && timestamp < System.currentTimeMillis());
                }
            }
        } catch (ACServiceException e) {
            e.printStackTrace();
            fail("test query history device data fail");
        }
    }

    @Test
    public void test3FetchHistoryPropertyData() throws Exception {
        if (account == null || light == null)
            return;

        // 先测试智能灯上报消息，将上报数据写入设备属性中
        try {
            ACDeviceMsg acDeviceMsg = new ACDeviceMsg(LightMsg.REPORT_CODE, new byte[]{LightMsg.ON, LightMsg.FROM_SWITCH});
            // 这里直接调用服务的设备消息处理handler，驱动测试
            ACDeviceReportInfo reportInfo = new ACDeviceReportInfo();
            reportInfo.setContext(ac.newContext(subDomainName));
            reportInfo.setDeviceId(light.getId());
            reportInfo.setPhysicalDeviceId(light.getPhysicalId());
            demoService.handleDeviceMsg(reportInfo, acDeviceMsg);
        } catch (ACServiceException e) {
            e.printStackTrace();
            fail("test light report fail");
        }

        try {
            // 创建一个用户的context
            ACContext context = ac.newContext(subDomainName, account.getUid());

            // 下面构造client发送的请求参数
            ACMsg req = new ACMsg();
            req.setContext(context);            // 将上面用户的context传入
            req.setName("fetchHistoryPropertyData");        // 设置请求消息的名字
            req.put("deviceId", light.getId()); // 设置要控制灯的逻辑id
            req.put("startTime", startTime); //查询历史属性数据开始时间
            req.put("endTime", System.currentTimeMillis()); //查询历史属性数据结束时间

            // 构造响应消息，用于接收服务端返回的消息
            ACMsg resp = new ACMsg();
            // 这里直接调用服务的处理handler，驱动测试
            demoService.handleMsg(req, resp);
            // 比较结果是否正确
            assertEquals("success", resp.get("result"));
        } catch (ACServiceException e) {
            e.printStackTrace();
            fail("test fetch history property data fail");
        }
    }
}
